from meterviewer.generator import single, db, total
from meterviewer import files
from pathlib import Path as P
import pytest

from meterviewer.models import littledb, func
from tests.utils import show_img


@pytest.mark.skip("会改变generated目录中的数据")
def test_total(root_path, alldata):
  item = func.get_first_item(alldata)
  total.cut_save_one(root_path, P(item.filename))


@pytest.mark.skip("这个函数会产生数据库 items.db，不应该直接测试")
def test_create_db(root_path):
  db.create_db(root_path)


def test_cut_one_img(root_path):
  # 裁剪所有图片
  img_filepath = root_path / r"lens_6/XL/XL/M1L1XL"
  img = next(files.scan_pics(img_filepath))
  im_list, v_list = total.cut_one_img(root_path / img)

  # might be changed when img changed.
  assert v_list[0] == "0"
  show_img(im_list[-1])


def test_generate_dbfiles(root_path):
  dataset_path = root_path / r"lens_6/XL/XL/M1L1XL"
  db_path = dataset_path / "items-temp.db"
  insert_one, _ = littledb.create_db(db_path)
  db.generate_for_one_dataset(dataset_path, insert_one)


def test_img_selector(root_path):
  get_random_img = single.img_selector(root_path)
  get_random_img("")


def test_single_digit(root_path):
  pass
  # gen = single.generate_single(root_path)
  # gen()



import streamlit as st
import pandas as pd
import pathlib
import numpy as np
import os
from meterviewer.config import get_root_path


def our_app():
  st.title("Quick Viewer of Meterdata")
  root_path = get_root_path()
  root_path = st.text_input("Enter the root path", value=str(root_path))
  root_path = pathlib.Path(root_path)

  path = st.text_input("Enter the path of the data", value="generated_merged")
  st.text(os.listdir(root_path / path))

  num = st.text_input("Enter the number of data")
  x_name = st.text_input("Enter the value of x_name", value="x_test.npy")
  y_name = st.text_input("Enter the value of y_name", value="y_test.npy")

  try:
    data_load_state = st.text("reading data...")
    num = int(num)
    data_load_state.text("Done!")
  except ValueError:
    data_load_state.text("Please enter a number")
    return

  folder_path = root_path / path
  x_path = folder_path / x_name
  y_path = folder_path / y_name

  x = np.load(x_path)
  y = np.load(y_path)
  st.text(f"x_shape: {x.shape}, y_shape: {y.shape}")

  st.image(x[num], caption=f"Meterdata {num}")
  st.text(f"Meterdata {num} is {y[num]}")


our_app()



# 统一数据集表示
# 只在一个文件里修改目录，方便快捷
# 将使用的函数放在参数上，减少依赖

from __future__ import annotations

import datetime
import os
import pathlib
import random
import typing as t

import numpy as np
from PIL import Image
from tqdm import tqdm

from ... import files
from ... import types as T
from ...img import process
from ..join import join_with_fix, join_with_resize  # noqa
from ..view import view_dataset, view_dataset_in_rows, view_dataset_on_disk  # noqa


def generate_func() -> t.List[t.Callable[[pathlib.Path], T.ImgDataset]]:
  """生成一组用于加载数据集的函数

  Returns:
      返回一个函数列表,每个函数用于加载指定路径下的训练或测试数据
  """
  funcs = []
  names: t.List[str] = [T.x_name, T.y_name, T.x_test, T.y_test]
  for n in names:

    def get_func(path: pathlib.Path, name: str = n):
      return files.load_from_disk(path / name)

    funcs.append(get_func)

  return funcs


def get_details(
  path: pathlib.Path,
  x: T.ImgDataset,
  y: T.LabelData,
) -> t.Dict:
  """获取数据集的详细信息

  Args:
      path: 数据集路径
      x: 图像数据集
      y: 标签数据

  Returns:
      包含数据集详细信息的字典,包括路径、创建时间、数据形状等
  """
  data = {}

  def create_sub(name):
    data[name] = {}

  for name in ["Dataset", "Meta"]:
    create_sub(name)

  data["Dataset"]["path"] = str(path)
  data["Meta"]["config.create_time"] = datetime.datetime.now()
  data["Dataset"]["created_time"] = ""
  data["Dataset"]["updated_time"] = ""
  data["Dataset"]["x_shape"] = x.shape
  data["Dataset"]["y_shape"] = y.shape
  return data


def show_details(
  get_x_train: t.Callable,
  get_y_train: t.Callable,
  get_details: t.Callable[[t.Any, t.Any], t.Dict],
  write_to_file: t.Callable[[t.Dict], None],
):
  x, y = get_x_train(), get_y_train()
  details = get_details(x, y)
  write_to_file(details)


def dataset_length_list() -> t.List[int]:
  """返回支持的数字长度列表

  Returns:
      支持的数字位数列表[5,6,7,8]
  """
  return [5, 6, 7, 8]


def create_str_with_fill(
  number: int,
  length: int,
  total: int,
) -> T.DigitStr:
  """创建填充后的数字字符串

  Args:
      number: 要转换的数字
      length: 数字的长度
      total: 需要填充到的总长度

  Returns:
      填充后的数字字符串
  """
  return fill_digit(process.number_to_string(number, length), total)


def create_labels_func(
  length: int,
  total: int,
) -> t.Callable[[int], t.Tuple[t.List[int], t.List[T.DigitStr]]]:
  """创建标签生成函数

  Args:
      length: 数字长度
      total: 填充后的总长度

  Returns:
      返回一个函数,该函数生成指定数量的随机数字及其字符串表示
  """

  def generate_nums(train_nums: int):
    numbers = []
    str_digits = []
    for _ in range(train_nums):
      number = random.randint(0, 10**length)
      numbers.append(number)
      str_digits.append(
        create_str_with_fill(number, length, total),
      )
    return numbers, str_digits

  return generate_nums


class GenBlockImgFunc(t.Protocol):
  def __call__(self, digit: T.DigitStr) -> T.NpImage: ...


class SaveDatasetFunc(t.Protocol):
  def __call__(self, imgs: T.ImgList, str_digits: t.List[T.DigitStr]) -> None: ...


class CreateDatasetFunc(t.Protocol):
  def __call__(
    self,
    length: int,
    nums: int,
    gen_block_img: GenBlockImgFunc,
  ) -> t.Tuple[t.List[T.NpImage], t.List[T.DigitStr]]: ...


def create_dataset_func(check_imgs: t.Callable[[T.ImgList], None]):
  def inner(
    root: pathlib.Path,
    dataset_list: t.List[str],
    available_digits: t.Dict[str, t.List[int]],  # 添加 available_digits 参数
    nums: int,
    gen_block_img: t.Callable[
      [pathlib.Path, t.List[str], t.Dict[str, t.List[int]], str],
      t.Tuple[T.NpImage, str],
    ],
    target_size: t.Tuple[int, int] = (297, 37),  # 目标尺寸
  ):
    imgs = []
    labels = []

    for _ in tqdm(range(nums)):
      digit = str(random.randint(0, 9))  # 随机选择数字
      img, new_label = gen_block_img(
        root, dataset_list, available_digits, digit
      )  # 传入 available_digits
      img_resized = Image.fromarray(img).resize(target_size)
      new_label_array = list(new_label)
      imgs.append(np.array(img_resized))
      labels.append(new_label_array)

    check_imgs(imgs)
    return imgs, labels

  return inner


def get_random_dataset(
  root: pathlib.Path, get_dataset_list: t.Callable
) -> t.Tuple[pathlib.Path, int]:
  """随机获取一个数据集

  Args:
      root: 根目录
      get_dataset_list: 获取数据集列表的函数

  Returns:
      随机选择的数据集路径和索引
  """
  datasets = list(get_dataset_list(root))
  random_index = random.randint(0, len(datasets) - 1)
  return datasets[random_index], random_index


def scan_available_digits(
  root: pathlib.Path, dataset_list: t.List[str]
) -> t.Dict[str, t.List[int]]:
  """
  扫描数据集中实际包含的数字。

  Args:
      root: 数据集根目录
      dataset_list: 数据集名称列表

  Returns:
      每个数据集中实际可用数字的字典。
  """
  available_digits = {}
  for dataset_name in dataset_list:
    digit_dir = root / "lens_6" / "XL" / "XL" / dataset_name / "Digit"
    available_digits[dataset_name] = [
      int(digit.name)
      for digit in digit_dir.iterdir()
      if digit.is_dir() and digit.name.isdigit()
    ]
  return available_digits


def generate_block_img(
  root: pathlib.Path,
  dataset_list: t.List[str],  # 数据集名称列表
  available_digits: t.Dict[str, t.List[int]],  # 每个数据集中可用的数字
  digit: str,  # 用于随机数字选择的参数（可忽略）
  target_size: t.Tuple[int, int] = (37, 297),  # 目标尺寸 (高度, 宽度)
  left_replace_width: int = 147,  # 替换区域的总宽度
) -> t.Tuple[T.NpImage, str]:
  """
  从多个数据集中随机选择图像 A 和三个数字图像 B、C、D，
  替换图像 A 的前三位数字，生成新的图像 E 和对应的标签。
  第一位和第二位数字替换逻辑调整为数字 0 占比 38%。

  Args:
      root: 数据集根目录
      dataset_list: 数据集名称列表
      available_digits: 每个数据集中可用的数字
      digit: 不使用，仅占位
      target_size: 目标尺寸
      left_replace_width: 替换区域总宽度

  Returns:
      新生成的图像 E 和对应的标签。
  """

  def select_digit_with_probability(
    digits: t.List[int], probabilities: t.List[float]
  ) -> int:
    """
    根据给定的概率分布从数字列表中选择一个数字。

    Args:
        digits: 数字列表
        probabilities: 对应的概率分布

    Returns:
        选择的数字
    """
    return random.choices(digits, weights=probabilities, k=1)[0]

  # 随机选择数据集用于图像 A
  dataset_name_a = random.choice(dataset_list)
  image_block_dir = (
    root / "lens_6" / "XL" / "XL" / dataset_name_a / "ImageSets_block_zoom"
  )
  block_imgs = list(image_block_dir.glob("*.jpg"))
  if not block_imgs:
    raise Exception(f"No images found in {image_block_dir}")
  img_a_path = random.choice(block_imgs)
  img_a = Image.open(img_a_path).convert("RGB")

  # 从文件名解析 A 的标签
  img_a_label = img_a_path.stem.split("_")[2]

  # 调整图像 A 到目标尺寸
  img_a = np.array(Image.fromarray(np.array(img_a)).resize(target_size[::-1]))

  # 定义数字选择规则
  digits = list(range(10))  # 数字 0-9
  probabilities = [0.38] + [0.62 / 9] * 9  # 数字 0 占 38%，其余数字均匀分布

  # 替换图像的数字
  replacements = []
  for i in range(3):
    while True:
      dataset_name_b = random.choice(dataset_list)
      digits_for_dataset = available_digits.get(dataset_name_b, [])
      if digits_for_dataset:
        if i == 0:  # 第一位数字
          digit = select_digit_with_probability(digits, probabilities)
        elif i == 1:  # 第二位数字
          digit = select_digit_with_probability(digits, probabilities)
        else:  # 第三位数字，保持随机
          digit = random.choice(digits_for_dataset)

        # 检查数字是否存在于数据集
        if digit in digits_for_dataset:
          digit_img_dir = (
            root / "lens_6" / "XL" / "XL" / dataset_name_b / "Digit" / str(digit)
          )
          digit_imgs = list(digit_img_dir.glob("*.jpg"))
          if digit_imgs:
            img_b_path = random.choice(digit_imgs)
            img_b = Image.open(img_b_path).convert("RGB")
            replacements.append((str(digit), np.array(img_b)))
            break

  # 替换图像 A 的前三位数字
  section_width = left_replace_width // 3  # 每个数字的替换宽度
  for i, (digit, img_b) in enumerate(replacements):
    img_b_resized = np.array(
      Image.fromarray(img_b).resize((section_width, target_size[0]))
    )
    img_a[:, i * section_width : (i + 1) * section_width, :] = img_b_resized

  # 更新标签
  new_label = "".join([digit for digit, _ in replacements]) + img_a_label[3:]

  # 确保替换后的图像 E 是目标尺寸
  img_a = np.array(Image.fromarray(img_a).resize(target_size[::-1]))

  return img_a, new_label


def get_dataset_path(root: pathlib.Path, dataset_name: str) -> pathlib.Path:
  """获取数据集路径

  Args:
      root: 根目录
      dataset_name: 数据集名称

  Returns:
      完整的数据集路径
  """
  p = root / "lens_6" / "XL" / "XL" / dataset_name
  return p


def get_dataset_list(
  root: pathlib.Path,
  default_func: t.Callable = lambda: pathlib.Path("lens_6/XL/XL"),
) -> t.Iterator[pathlib.Path]:
  """获取数据集列表

  Args:
      root: 根目录
      default_func: 默认路径生成函数

  Returns:
      数据集路径的迭代器
  """
  root = root / default_func()
  for dir in os.listdir(root):
    if os.path.isdir(root / dir):
      yield pathlib.Path(dir)


def handle_datasets(root: pathlib.Path, handle_func: t.Callable[[pathlib.Path], None]):
  """handle"""
  for dataset in get_dataset_list(root):
    handle_func(dataset)


def fill_digit(digit: T.DigitStr, total_length: int) -> T.DigitStr:
  """填充数字字符串到指定长度

  Args:
      digit: 原始数字字符串
      total_length: 目标长度

  Returns:
      填充后的数字字符串

  Raises:
      ValueError: 当输入字符串长度小于5或大于目标长度时
  """
  digit2 = digit.copy()
  if len(digit) < 5:
    raise ValueError(f"digit lenght must > 5, {digit}")

  if len(digit) <= total_length:
    digit2.extend(["x"] * (total_length - len(digit)))
  else:
    raise ValueError(f"digit lenght must < {total_length}, {digit}")
  return digit2



import numpy as np

from . import cmp


def test_two_data():
  im1 = np.random.rand(100, 100, 3) * 255
  im2 = im1.copy()

  im3 = np.random.rand(100, 100, 3) * 255
  assert cmp.comp_ims(im1, im2)
  assert not cmp.comp_ims(im1, im3)


def test_one_chanel():
  im1 = np.random.rand(100, 100, 1) * 255
  im2 = im1.copy()

  # imarray = np.random.rand(100,100,1) * 255
  assert cmp.comp_ims(im1, im2)



# quick view of a dataset

import streamlit as st
import pandas as pd
import numpy as np
from matplotlib import pyplot as plt
from meterviewer.config import get_root_path
from meterviewer.datasets import imgv
from meterviewer.img import draw, cut
import pathlib


def main():
  st.title("Quick Viewer of Meter dataset")
  data_path = pathlib.Path("/home/xiuhao/work/Dataset/MeterData/lens_5/XL/DATA")

  dataset_len_ = st.number_input("Dataset length", value=6)
  dataset_name = st.text_input("Dataset name", value="M4L1XL")
  filename = st.text_input("Enter the filename", value="2018-11-23-12-16-01.jpg")

  img_path = data_path / dataset_name / filename
  im, v, rect = imgv.view_one_img_v(img_path)

  st.text(f"image shape: {im.shape}")
  st.image(im, caption=f"filename: {filename}")

  st.text("with rect")
  rect_im = draw.draw_rectangle(im, rect)
  st.image(rect_im, caption=f"with rect: {filename}")

  cutted = cut.cut_img(im, rect)
  st.image(cutted, caption=f"cutted: {filename}")


main()



import streamlit as st
import pandas as pd
import numpy as np
from matplotlib import pyplot as plt
from meterviewer.config import get_root_path
from meterviewer.datasets import imgv
from meterviewer.img import draw, cut
import pathlib


def example():
  st.title("Uber pickups in NYC")
  DATE_COLUMN = "date/time"
  DATA_URL = (
    "https://s3-us-west-2.amazonaws.com/"
    "streamlit-demo-data/uber-raw-data-sep14.csv.gz"
  )

  @st.cache_data
  def load_data(nrows):
    data = pd.read_csv(DATA_URL, nrows=nrows)
    lowercase = lambda x: str(x).lower()
    data.rename(lowercase, axis="columns", inplace=True)
    data[DATE_COLUMN] = pd.to_datetime(data[DATE_COLUMN])
    return data

  data_load_state = st.text("Loading data...")
  data = load_data(10000)
  data_load_state.text("Done! (using st.cache_data)")

  if st.checkbox("Show raw data"):
    st.subheader("Raw data")
    st.write(data)

  st.subheader("Number of pickups by hour")
  hist_values = np.histogram(data[DATE_COLUMN].dt.hour, bins=24, range=(0, 24))[0]
  st.bar_chart(hist_values)

  # Some number in the range 0-23
  hour_to_filter = st.slider("hour", 0, 23, 17)
  filtered_data = data[data[DATE_COLUMN].dt.hour == hour_to_filter]

  st.subheader("Map of all pickups at %s:00" % hour_to_filter)
  st.map(filtered_data)



import sys
import pathlib
import importlib
import toml

common_config = {
    'loglevel': 'INFO'
}

def load_env():
    parent = pathlib.Path("./playground")
    c = parent / "dataset.toml"
    content = c.read_text()
    data = toml.loads(content)
    base_config = data.get('base', None)
    common_config['loglevel'] = base_config.get('loglevel', 'INFO')


def main():
    load_env()
    mode = sys.argv[1]
    module = importlib.import_module(mode)
    module.main()


if __name__ in ("__main__"):
    main()



import toml
import pathlib
import typing as t
from nicegui import ui
from nicegui.events import ValueChangeEventArguments, KeyEventArguments
from meterviewer.config import get_root_path
from meterviewer.labeling import views


def get_image_path(
  root_path: pathlib.Path, mode: t.Literal["filesys", "db", "quick_view"]
):
  if mode == "filesys":
    return views.filesystem.from_filesystem(root_path=root_path)
  elif mode == "quick_view":
    return views.filesystem.view_dataset(root_path=root_path)
  elif mode == "db":
    return views.db.from_db()
  else:
    raise ValueError("mode not found.")


def show(event: ValueChangeEventArguments):
  name = type(event.sender).__name__
  ui.notify(f"{name}: {event.value}")


def old_layout():
  ui.button("Button", on_click=lambda: ui.notify("Click"))
  with ui.row():
    ui.checkbox("Checkbox", on_change=show)
    ui.switch("Switch", on_change=show)
  ui.radio(["A", "B", "C"], value="A", on_change=show).props("inline")
  with ui.row():
    ui.input("Text input", on_change=show)
    ui.select(["One", "Two"], value="One", on_change=show)


def main():
  ui.markdown("# Labeling tool for meter image.")

  root_path = get_root_path()

  def handle_key(e: KeyEventArguments):
    if e.key == "Enter":
      if e.action.keyup:
        set_img_value()

  _ = ui.keyboard(handle_key)

  dataset_g = get_image_path(root_path=root_path, mode="quick_view")
  img_path, dataset_name, id = next(dataset_g)
  carry_v = None

  def set_img_value():
    nonlocal carry_v, img_path, id, dataset_name
    if carry_v is None:
      return
    print(f"image_name: {img_path}")
    print(f"value_set: {carry_v}")
    img_path, dataset_name, id = next(dataset_g)
    current_img.source = img_path
    mrk.content = f"dataset: {dataset_name}, count: {id}/20222"
    toggle1.set_value(None)

  # make sure which dataset to use.
  collect_set = set()

  def add_data(value):
    if value == "1" or value == 1:
      collect_set.add(dataset_name)

  def write_data():
    with open("temp.toml", "w") as f:
      toml.dump({"dataset": list(collect_set)}, f)

  def set_value(event: ValueChangeEventArguments):
    nonlocal carry_v
    carry_v = event.value
    add_data(carry_v)
    set_img_value()

  mrk = ui.markdown(f"dataset: {dataset_name}, count: 1/20222")
  with ui.row():
    current_img = ui.image(img_path).classes("w-[540px]")

  with ui.row():
    toggle1 = ui.toggle([0, 1, 2, 3, 4, 5, 6], value=carry_v, on_change=set_value)
    ui.input("Carry digit", on_change=set_value)
  with ui.row():
    ui.button("previous")
    ui.button("confirm", on_click=lambda: set_img_value())
    ui.button("write", on_click=write_data)
  # ui.link("And many more...", "/documentation").classes("mt-8")

  ui.run()


if __name__ in {"__main__", "__mp_main__"}:
  main()



# generate dataset path.
import os
import pathlib
from meterviewer.config import get_root_path
import streamlit as st


def main():
  p = get_root_path()
  p = pathlib.Path(st.text_input("root path:", value=p))

  st.text("choose your dataset")
  st.text(os.listdir(p))
  dataset_name = st.text_input("dataset name")
  ds_path = p / dataset_name
  st.button("finish")
  st.text(ds_path)


main()



# generate single digit
from __future__ import annotations

import pathlib
import typing as t
from datetime import datetime
from pathlib import Path as P

import numpy as np

from meterviewer import T, files
from meterviewer.datasets import dataset
from meterviewer.img import process


def use_write(
  im: T.NpImage, filename: str
) -> t.Callable[[P, int, t.Literal["normal", "digit"]], None]:
  """
  Create a function that writes an image to a specific location.

  Args:
      im (T.NpImage): The image to be written.
      filename (str): The name of the file to be created.

  Returns:
      Callable: A function that writes the image to a specified path.
  """

  def write_to(
    root_path: P,
    digit: int,
    type_: t.Literal["normal", "digit"],
  ):
    """
    Write the image to a specific location based on the provided parameters.

    Args:
        root_path (P): The root path where the dataset will be created.
        digit (int): The digit represented in the image.
        type_ (Literal["normal", "digit"]): The type of image (normal or digit).
    """
    # Get current date for dataset naming
    date = datetime.now().strftime("%Y-%m-%d")
    dataset_path = root_path / f"generated-{date}"
    digit_path = dataset_path / type_ / str(digit)

    # Create directory if it doesn't exist
    digit_path.mkdir(parents=True, exist_ok=True)

    # Save the image as a numpy array
    np.save(digit_path / filename, im)

  return write_to


def generate_total_dataset(
  length: int,
  size: int,
  create_np: t.Callable[[T.ImgList, t.List[t.List[int]]], None],
):
  """size: dataset samples size"""

  def get_random_number():
    pass

  def get_one_img(digit: int):
    pass

  def join_with(imglist: T.ImgList):
    # this join-with will check the size
    return process.join_img(imglist, process.size_check)

  def multiple_gen(get_random_number: t.Callable, get_one_img: t.Callable):
    img_list = []
    v_list = []
    for _ in range(size):
      gen_func = generate_block(length, get_random_number)
      im, v = gen_func(get_one_img=get_one_img, join_img=join_with)
      img_list.append(im)
      v_list.append(v)

    return create_np(img_list, v_list)

  return multiple_gen


def generate_block(length: int, get_random_number: t.Callable[[int], t.List[int]]):
  def gen(
    get_one_img: t.Callable[[int], T.NpImage],
    join_img: t.Callable[[T.ImgList], T.NpImage],
  ):
    """generate single digit image"""

    number: t.List[int] = get_random_number(length)
    assert len(number) == length

    img_list = []
    for digit in number:
      img = get_one_img(digit)
      img_list.append(img)

    im = join_img(img_list)
    return im, number

  return gen


def img_selector(root_path: P):
  def get_random_img(
    dataset_name: str,
  ) -> t.Tuple[T.ImgList, t.List[str], t.Literal["normal", "digit"]]:
    dp = dataset.get_dataset_path(root_path, dataset_name)

  return get_random_img


def generate_single(
  root_path: pathlib.Path,
  get_random_img: t.Callable[
    [str], t.Tuple[T.ImgList, t.List[str], t.Literal["normal", "digit"]]
  ],
):
  """generate single digit image, split to normal and digit folder"""

  def handle_img(
    im: T.NpImage,
    filename: str,
    digit: int,
    type_: t.Literal["normal", "digit"],
  ):
    """write to specific folder"""
    write = use_write(im, filename)
    write(root_path, digit, type_)

  def inner(filename):
    def handle_dataset(dataset_name: str | pathlib.Path):
      imgs, values, types_ = get_random_img(str(dataset_name))
      for im, v in zip(imgs, values):
        handle_img(im, v, filename, types_)

    dataset.handle_datasets(root_path, handle_dataset)

  return inner



from playground.generate_db import create_all


def test_create(root_path):
    # this script is used for only once.
    if (root_path / "alldata.db").exists():
        return
    create_all.main(root_path)



# 该模块用于管理数据集文件的命名规则
import typing as t

from meterviewer import types as T


def normal_x_y() -> t.Tuple[str, str]:
  """返回普通数据集的X和Y文件名

  Returns:
      tuple: 包含X数据文件名和Y数据文件名的元组
  """
  return "x.npy", "y.npy"


def train_x_y() -> t.Tuple[str, str]:
  """返回训练数据集的X和Y文件名

  Returns:
      tuple: 包含X训练数据文件名和Y训练数据文件名的元组
  """
  return T.x_name, T.y_name


# 文件命名函数的映射字典
name_funcs = {
  "normal": normal_x_y,  # 普通数据集命名函数
  "train": train_x_y,  # 训练数据集命名函数
}



import h5py
import pathlib


def test_h5py():
    path = pathlib.Path("/home/svtter/Work/Dataset/MeterData/generated_merged")
    f = h5py.File(path / "generated.hdf5", "r")
    content = []
    for name, value in f.items():
        content.append(name)



import pathlib

import pytest

from meterviewer.datasets.read import detection

sample_dataset = "M1L3XL"


def select_one(root_path, dataset_name: str) -> pathlib.Path:
  """第一个图片"""
  return pathlib.Path(detection.list_images(root_path, dataset_name)[0])


def test_read_to_get(root_path):
  res = detection.read_image_area(
    select_one(root_path, sample_dataset),
  )
  assert len(res) > 0


def test_list_imgs(root_path):
  assert len(detection.list_images(root_path, "M1L3XL")) > 0



"""
create a little db for storing the images and values.

some basic models.
"""

from __future__ import annotations

import pathlib
import typing as t
from typing import List, Optional

from sqlalchemy import ForeignKey, String, create_engine
from sqlalchemy.orm import DeclarativeBase, Mapped, Session, mapped_column, relationship


def get_session(filepath: str | pathlib.Path):
  filepath = str(filepath)
  engine = create_engine(f"sqlite:///{filepath}", echo=True)
  with Session(engine) as session:
    return session


def create_db(filepath: str):
  engine = create_engine(f"sqlite:///{filepath}", echo=True)
  Base.metadata.create_all(engine)

  with Session(engine) as session:

    def insert_one(filename: str, value: int, is_carry: bool):
      item = Item(filename=filename, value=value, is_carry=is_carry)
      session.add(item)
      session.commit()

    def insert_all(items: t.List[Item]):
      session.add_all(items)
      session.commit()

    return insert_one, insert_all


class Base(DeclarativeBase):
  pass


class Item(Base):
  __tablename__ = "item"
  id: Mapped[int] = mapped_column(primary_key=True)
  filename: Mapped[str] = mapped_column(String(30))
  value: Mapped[int] = mapped_column()
  is_carry: Mapped[bool] = mapped_column()
  carry_num: Mapped[int] = mapped_column()

  def __repr__(self) -> str:
    return f"Item(id={self.id!r}, filename={self.filename!r}, value={self.value!r})"


# examples
def examples():
  class User(Base):
    __tablename__ = "user_account"
    id: Mapped[int] = mapped_column(primary_key=True)
    name: Mapped[str] = mapped_column(String(30))
    fullname: Mapped[Optional[str]]
    addresses: Mapped[List["Address"]] = relationship(
      back_populates="user", cascade="all, delete-orphan"
    )

    def __repr__(self) -> str:
      return f"User(id={self.id!r}, name={self.name!r}, fullname={self.fullname!r})"

  class Address(Base):
    __tablename__ = "address"
    id: Mapped[int] = mapped_column(primary_key=True)
    email_address: Mapped[str]
    user_id: Mapped[int] = mapped_column(ForeignKey("user_account.id"))
    user: Mapped["User"] = relationship(back_populates="addresses")

    def __repr__(self) -> str:
      return f"Address(id={self.id!r}, email_address={self.email_address!r})"



# import cv2
from meterviewer import types as T


def cut_img(img: T.NpImage, rect: T.Rect) -> T.NpImage:
  def to_int(rect: T.Rect):
    return (
      int(rect["xmin"]),
      int(rect["ymin"]),
      int(rect["xmax"]),
      int(rect["ymax"]),
    )

  def get_cut_region(rect: T.Rect):
    x_min, y_min, x_max, y_max = to_int(rect)
    x, y, w, h = x_min, y_min, x_max - x_min, y_max - y_min
    return x, y, w, h

  x_min, y_min, x_max, y_max = to_int(rect)
  cropped_image = img[y_min:y_max, x_min:x_max]
  return cropped_image



# test fast ui.
from datetime import date
from fastapi import FastAPI, HTTPException
from fastapi.responses import HTMLResponse
from fastui import FastUI, AnyComponent, prebuilt_html, components as c
from fastui.components.display import DisplayMode, DisplayLookup
from fastui.events import GoToEvent, BackEvent
from pydantic import BaseModel, Field


class User(BaseModel):
    id: int
    name: str
    dob: date = Field(title="Date of Birth")


users = [
    User(id=1, name="John", dob=date(1990, 1, 1)),
    User(id=2, name="Jack", dob=date(1991, 1, 1)),
    User(id=3, name="Jill", dob=date(1992, 1, 1)),
    User(id=4, name="Jane", dob=date(1993, 1, 1)),
]


app = FastAPI()


@app.get("/api/", response_model=FastUI, response_model_exclude_none=True)
def users_table() -> list[AnyComponent]:
    """
    Show a table of four users, `/api` is the endpoint the frontend will connect to
    when a user visits `/` to fetch components to render.
    """
    return [
        c.Page(  # Page provides a basic container for components
            components=[
                c.Heading(text="Users", level=2),  # renders `<h2>Users</h2>`
                c.Table(
                    data=users,
                    # define two columns for the table
                    columns=[
                        # the first is the users, name rendered as a link to their profile
                        DisplayLookup(
                            field="name", on_click=GoToEvent(url="/user/{id}/")
                        ),
                        # the second is the date of birth, rendered as a date
                        DisplayLookup(field="dob", mode=DisplayMode.date),
                    ],
                ),
            ]
        ),
    ]


@app.get(
    "/api/user/{user_id}/", response_model=FastUI, response_model_exclude_none=True
)
def user_profile(user_id: int) -> list[AnyComponent]:
    """
    User profile page, the frontend will fetch this when the user visits `/user/{id}/`.
    """
    try:
        user = next(u for u in users if u.id == user_id)
    except StopIteration:
        raise HTTPException(status_code=404, detail="User not found")
    return [
        c.Page(
            components=[
                c.Heading(text=user.name, level=2),
                c.Link(components=[c.Text(text="Back")], on_click=BackEvent()),
                c.Details(data=user),
            ]
        ),
    ]


@app.get("/{path:path}")
async def html_landing() -> HTMLResponse:
    """Simple HTML page which serves the React app, comes last as it matches all paths."""
    return HTMLResponse(prebuilt_html(title="FastUI Demo"))



import numpy as np
from meterviewer.img import validator


def test_validator():
  v = validator.Validator()

  cases = [
    (np.array([1, 2, 3]), False),
  ]
  for img, expected in cases:
    assert expected == v.is_img(img)



import ast
import os
import pathlib

import numpy as np
import pandas as pd
import streamlit as st
from meterviewer.config import get_root_path
from meterviewer.img import resize


def resize_app():
  st.title("Tool to resize images.")
  root_path = pathlib.Path("/home/xiuhao/work/Dataset/HRCdata")
  root_path = st.text_input("Enter the root path", value=str(root_path))
  root_path = pathlib.Path(root_path)

  path = st.text_input("Enter the path of the data", value="Train")
  st.text(os.listdir(root_path / path))

  x_name = st.text_input("Enter the value of x_name", value="x_train.original.npy")
  y_name = st.text_input("Enter the value of y_name", value="y_train.npy")
  output_name = st.text_input("Enter the output name", value="x_train.npy")

  folder_path = root_path / path
  x_path = folder_path / x_name
  y_path = folder_path / y_name

  x = np.load(x_path)
  y = np.load(y_path)
  st.text(f"x_shape: {x.shape}, y_shape: {y.shape}")

  size = st.text_input("Enter the new size of the image", value="(32, 300)")
  size = ast.literal_eval(size)
  if st.button("transform"):
    transform_state = st.text("transforming...")
    res = resize.resize_imglist(x, size)
    np.save(folder_path / output_name, res)
    transform_state.text("transformed!")


resize_app()



# 统一数据集表示
# 只在一个文件里修改目录，方便快捷
# 将使用的函数放在参数上，减少依赖

from __future__ import annotations

import datetime
import os
import pathlib

# import numpy as np
import random
import typing as t

from tqdm import tqdm

from ... import files
from ... import types as T
from ...img import process
from ..join import join_with_fix, join_with_resize  # noqa
from ..view import view_dataset, view_dataset_in_rows, view_dataset_on_disk  # noqa


def generate_func() -> t.List[t.Callable[[pathlib.Path], T.ImgDataset]]:
  """生成一组用于加载数据集的函数

  Returns:
      返回一个函数列表,每个函数用于加载指定路径下的训练或测试数据
  """
  funcs = []
  names: t.List[str] = [T.x_name, T.y_name, T.x_test, T.y_test]
  for n in names:

    def get_func(path: pathlib.Path, name: str = n):
      return files.load_from_disk(path / name)

    funcs.append(get_func)

  return funcs


def get_details(
  path: pathlib.Path,
  x: T.ImgDataset,
  y: T.LabelData,
) -> t.Dict:
  """获取数据集的详细信息

  Args:
      path: 数据集路径
      x: 图像数据集
      y: 标签数据

  Returns:
      包含数据集详细信息的字典,包括路径、创建时间、数据形状等
  """
  data = {}

  def create_sub(name):
    data[name] = {}

  for name in ["Dataset", "Meta"]:
    create_sub(name)

  data["Dataset"]["path"] = str(path)
  data["Meta"]["config.create_time"] = datetime.datetime.now()
  data["Dataset"]["created_time"] = ""
  data["Dataset"]["updated_time"] = ""
  data["Dataset"]["x_shape"] = x.shape
  data["Dataset"]["y_shape"] = y.shape
  return data


def show_details(
  get_x_train,
  get_y_train,
  get_details: t.Callable[[t.Any, t.Any], t.Dict],
  write_to_file: t.Callable[[t.Dict], None],
):
  x, y = get_x_train(), get_y_train()
  details = get_details(x, y)
  write_to_file(details)


def dataset_length_list() -> t.List[int]:
  """返回支持的数字长度列表

  Returns:
      支持的数字位数列表[5,6,7,8]
  """
  return [5, 6, 7, 8]


def create_str_with_fill(
  number: int,
  length: int,
  total: int,
) -> T.DigitStr:
  """创建填充后的数字字符串

  Args:
      number: 要转换的数字
      length: 数字的长度
      total: 需要填充到的总长度

  Returns:
      填充后的数字字符串
  """
  return fill_digit(process.number_to_string(number, length), total)


def create_labels_func(
  length: int,
  total: int,
) -> t.Callable[[int], t.Tuple[t.List[int], t.List[T.DigitStr]]]:
  """创建标签生成函数

  Args:
      length: 数字长度
      total: 填充后的总长度

  Returns:
      返回一个函数,该函数生成指定数量的随机数字及其字符串表示
  """

  def generate_nums(train_nums: int):
    numbers = []
    str_digits = []
    for _ in range(train_nums):
      number = random.randint(0, 10**length)
      numbers.append(number)
      str_digits.append(
        create_str_with_fill(number, length, total),
      )
    return numbers, str_digits

  return generate_nums


class GenBlockImgFunc(t.Protocol):
  def __call__(self, digit: T.DigitStr) -> T.NpImage: ...


class SaveDatasetFunc(t.Protocol):
  def __call__(self, imgs: T.ImgList, str_digits: t.List[T.DigitStr]) -> None: ...


class CreateDatasetFunc(t.Protocol):
  def __call__(
    self,
    length: int,
    nums: int,
    gen_block_img: GenBlockImgFunc,
  ) -> t.Tuple[t.List[T.NpImage], t.List[T.DigitStr]]: ...


def create_dataset_func(
  check_imgs: t.Callable[[T.ImgList], None],
  total: int,
) -> CreateDatasetFunc:
  """创建数据集生成函数

  Args:
      check_imgs: 图像检查函数
      total: 填充后的总长度

  Returns:
      返回一个函数,用于生成包含指定数量和长度的数字图像数据集
  """

  def inner(
    length: int,
    nums: int,
    gen_block_img: GenBlockImgFunc,
  ):
    _, str_digits = create_labels_func(length, total)(nums)

    imgs = []
    for digit in tqdm(str_digits):
      im = gen_block_img(digit)
      imgs.append(im)

    # files.write_shape(imgs, 3)
    check_imgs(imgs)

    # automatic resize the images
    imgs = process.resize_imglist(imgs, size=[37, 297])
    return imgs, str_digits

  return inner


def get_random_dataset(
  root: pathlib.Path, get_dataset_list: t.Callable
) -> t.Tuple[pathlib.Path, int]:
  """随机获取一个数据集

  Args:
      root: 根目录
      get_dataset_list: 获取数据集列表的函数

  Returns:
      随机选择的数据集路径和索引
  """
  datasets = list(get_dataset_list(root))
  random_index = random.randint(0, len(datasets) - 1)
  return datasets[random_index], random_index


def generate_block_img(
  the_digit: T.DigitStr,
  join_func: T.JoinFunc,
  read_rand_img: t.Callable[[int | str], T.NpImage],
) -> T.NpImage:
  """生成数字块图像

  Args:
      the_digit: 数字字符串
      join_func: 图像拼接函数
      read_rand_img: 读取随机图像的函数

  Returns:
      拼接后的数字块图像
  """
  img_list = []
  for digit in the_digit:
    im = read_rand_img(digit)
    img_list.append(im)
  return join_func(img_list, process.size_check)


def get_dataset_path(root: pathlib.Path, dataset_name: str) -> pathlib.Path:
  """获取数据集路径

  Args:
      root: 根目录
      dataset_name: 数据集名称

  Returns:
      完整的数据集路径
  """
  p = root / "lens_6" / "XL" / "XL" / dataset_name
  return p


def get_dataset_list(
  root: pathlib.Path,
  default_func: t.Callable = lambda: pathlib.Path("lens_6/XL/XL"),
) -> t.Iterator[pathlib.Path]:
  """获取数据集列表

  Args:
      root: 根目录
      default_func: 默认路径生成函数

  Returns:
      数据集路径的迭代器
  """
  root = root / default_func()
  for dir in os.listdir(root):
    if os.path.isdir(root / dir):
      yield pathlib.Path(dir)


def handle_datasets(root: pathlib.Path, handle_func: t.Callable[[pathlib.Path], None]):
  """handle"""
  for dataset in get_dataset_list(root):
    handle_func(dataset)


def fill_digit(digit: T.DigitStr, total_length: int) -> T.DigitStr:
  """填充数字字符串到指定长度

  Args:
      digit: 原始数字字符串
      total_length: 目标长度

  Returns:
      填充后的数字字符串

  Raises:
      ValueError: 当输入字符串长度小于5或大于目标长度时
  """
  digit2 = digit.copy()
  if len(digit) < 5:
    raise ValueError(f"digit lenght must > 5, {digit}")

  if len(digit) <= total_length:
    digit2.extend(["x"] * (total_length - len(digit)))
  else:
    raise ValueError(f"digit lenght must < {total_length}, {digit}")
  return digit2



from meterviewer.img import draw, cut, process
from meterviewer.datasets import imgv
from meterviewer import types as T
from pathlib import Path as P


def test_cut(root_path):
  img_path = root_path / "lens_6/XL/XL/M1L3XL/2018-11-23-12-16-01.jpg"
  im = process.plt.imread(img_path)

  """<xmin>82</xmin>
			<ymin>39</ymin>
			<xmax>112</xmax>
			<ymax>72</ymax>"""

  rect = T.Rect(xmin="82", ymin="39", xmax="112", ymax="72")
  cut_img = cut.cut_img(im, rect)
  process.show_img(cut_img, is_stop=0)


def test_draw(root_path):
  img_path = root_path / "lens_6/XL/XL/M1L3XL/2018-11-23-12-16-01.jpg"
  im, v, rect = imgv.view_one_img_v(img_path)
  im = draw.draw_rectangle(im, rect)
  process.show_img(im, is_stop=0)


def test_draw_text(root_path):
  img_path = root_path / "lens_6/XL/XL/M1L3XL/2018-11-23-12-16-01.jpg"
  im, v, _ = imgv.view_one_img_v(img_path)
  im = draw.draw_text(im, v)
  process.show_img(im, is_stop=0)



import typing as t
import pathlib
from pathlib import Path

viewReturn = t.Generator[t.Tuple[pathlib.Path, pathlib.Path, str], None, None]

FullPath = Path


def isFullPath(p: FullPath):
  return p.is_absolute()
